import { _base_, setRoot, copyFromGit, setPWA } from './core';

import request from 'request-promise-native';

import { parse, stringify } from 'yaml';

import { join } from 'path';

import {
    ensureDirSync, writeJSON, readJSON, remove, existsSync, readdirSync, move,
    readFile, outputFile
} from 'fs-extra';

import { step, spawn, ensureCommand } from '@tech_query/node-toolkit';

import { bootGit } from 'create-es-pack/dist/core';


/**
 * @param {String} type - `plugin` or `theme`
 */
export  async function update(type) {

    const list = parse(await request(
        `https://raw.githubusercontent.com/hexojs/site/master/source/_data/${type}s.yml`
    ));

    ensureDirSync( join(_base_, 'data') );

    await writeJSON(join(_base_, `data/${type}.json`),  list,  {spaces: 4});

    console.info(`[ Update ]  ${list.length} ${type}s`);
}


/**
 * @param {String}   cwd  - Root path of a Wiki-site
 * @param {String}   type - `plugin` or `theme`
 * @param {String[]} name - Package name (Common prefix is optional)
 *
 * @return {String[]} Names of installed packages
 */
export  async function install(cwd, type, name) {

    const list = await readJSON( join(_base_, `data/${type}.json`) );

    const NPM = [ ],  Git = [ ],  command_option = {stdio: 'inherit', cwd};

    list.forEach(item => {

        if (! name.find(
            key  =>  RegExp(`^(hexo-(theme-)?)?${key}$`, 'i').test( item.name )
        ))
            return;

        if ( item.link.includes('npmjs.com/') )
            NPM.push( item.name );
        else if (type === 'plugin')
            NPM.push( item.link );
        else
            Git.push( item );
    });

    if ( NPM[0] )
        await spawn('npm',  ['install'].concat( NPM ),  command_option);

    if ( Git[0] )
        for (let item of Git)
            await copyFromGit(item.link,  join(cwd, `themes/${item.name}/`));

    return  NPM.concat( Git.map(item => item.name) );
}


async function addTheme(cwd, name, config) {

    name = (await install(cwd, 'theme', [name]))[0];

    if (! name)
        return console.error(
            `Theme "${name}" can't be found in Hexo offical index`
        );

    const path = join(cwd, `themes/${name}`);

    if (! existsSync(`${path}/_config.yml`))
        for (let file  of  readdirSync( path ))
            if (/^_config\..*yml.*/.test( file )) {

                await move(`${path}/${file}`, `${path}/_config.yml`);  break;
            }

    await remove( join(cwd, 'themes/landscape') );

    return  config.replace(/^theme:.+/m,  `theme: ${name}`);
}

/**
 * Boot a directory as a WebCell project
 *
 * @param {Object}     option
 * @param {String}     [option.cwd='.']    - Current working directory
 * @param {String[]}   [option.plugins=[]]
 * @param {?String}    option.theme
 * @param {String[]}   [option.pages=[]]   - Required pages of `theme`
 * @param {String|URL} [option.remote]     - Git URL of a Remote repository
 */
export  async function boot({
    cwd = '.',  plugins = [ ],  theme,  pages = [ ],  remote
} = { }) {
    const command_option = {stdio: 'inherit', cwd},
        config_path = join(cwd, '_config.yml');

    await step('Hexo framework',  async () => {

        await ensureCommand('hexo');

        await copyFromGit('https://github.com/hexojs/hexo-starter.git', cwd);
    });

    var config, git;

    await step('Git repository & branch',  async () => {

        config = await readFile(config_path)  +  '',
        git = await bootGit(cwd, remote);

        await git.raw(['checkout', '--orphan', 'hexo']);

        config = config.replace(/\ndeploy:[\s\S]+/, `
deploy:
  type: git
  repo: ${(await git.getRemotes( true ))[0].refs.push}
  branch: master`
        );
    });

    await step('NPM package',  setRoot.bind(null, cwd, git));

    await step('Hexo plugins',  async () => {

        const meta = parse( config );

        config += '\n\n' + stringify({
            pwa:  setPWA({
                name:         meta.title,
                description:  meta.subtitle || meta.description,
                lang:         meta.language,
                start_url:    meta.url,
                scope:        meta.root,
                icon:         {
                    src:    'image/Hexo.png',
                    type:   'image/png',
                    sizes:  '128x128'
                }
            })
        });

        await spawn('npm',  ['install'],  command_option);

        if ( plugins[0] )  await install(cwd, 'plugin', plugins);
    });

    await step('Hexo theme',  async () => {

        if ( theme )  config = await addTheme(cwd, theme, config) || config;

        for (let name of pages) {

            const file = join(cwd, 'source', name, 'index.md');

            await outputFile(file, `---
type: ${name}
layout: ${name}
---\n`
            );

            console.info(`[ Create ]  ${file}`);
        }

        await outputFile(config_path,  config);
    });

    await step('Git commit',  async () => {

        await git.add('.');

        await git.commit('[ Add ]  Framework of Hexo wiki');
    });
}
